@@ -12,13 +12,14 @@ package Data::Tools;
 use strict;
 use Exporter;
 use Carp;
+use Digest;
 use Digest::Whirlpool;
 use Digest::MD5;
 use Digest::SHA1;
 use File::Glob;
 use Hash::Util qw( lock_hashref unlock_hashref lock_ref_keys );
 
-our $VERSION = '1.12';
+our $VERSION = '1.14';
 
 our @ISA    = qw( Exporter );
 our @EXPORT = qw(
@@ -61,6 +62,10 @@ our @EXPORT = qw(
               
               str_hex 
               str_unhex
+
+              str_num_comma
+              str_pad
+              str_countable
               
               perl_package_to_file
 
@@ -68,6 +73,8 @@ our @EXPORT = qw(
               md5_hex
               sha1_hex
               
+              create_random_id
+              
               glob_tree
               read_dir_entries
 
@@ -239,6 +246,40 @@ sub str_unhex
 
 ##############################################################################
 
+sub str_num_comma
+{
+  my $data = shift;
+  $data = reverse $data;
+  $data =~ s/(\d\d\d)(?=\d)(?!\d*\.)/$1`/g;
+  $data = reverse $data;
+  return $data;
+}
+
+sub str_pad
+{
+  my $str = shift;
+  my $len = shift;
+  my $pad = shift;
+  $pad = ' ' unless defined $pad;
+
+  $str = reverse $str if $len < 0;
+  $str = substr( $str . ($pad x abs($len)), 0, abs($len) );
+  $str = reverse $str if $len < 0;
+
+  return $str;
+}
+
+sub str_countable
+{
+  my $count = shift;
+  my $one   = shift;
+  my $many  = shift;
+
+  return $count == 0 ? $many : $count == 1 ? $one : $many;
+}
+
+##############################################################################
+
 sub str2hash
 {
   my $str = shift;
@@ -487,6 +528,19 @@ sub sha1_hex
 
 ##############################################################################
 
+sub create_random_id
+{
+  my $len = shift() || 128;
+  my $let = shift() || 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
+
+  my $l = length( $let );
+  my $id;
+  $id .= substr( $let, int(rand() * $l), 1 ) for ( 1 .. $len );
+  return $id;
+};
+
+##############################################################################
+
 sub __glob_tree_tree_walk
 {
   my $p = shift; # path
@@ -641,6 +695,15 @@ INIT  { __url_escapes_init(); }
 
   # --------------------------------------------------------------------------
 
+  my $formatted_str = str_num_comma( 1234567.89 );  # returns "1'234'567.89"
+  my $padded_str    = str_pad( 'right', -12, '*' ); # returns "right*******"
+  my $str_c         = str_countable( $dc, 'day', 'days' );
+                      # returns 'days' for $dc == 0
+                      # returns 'day'  for $dc == 1
+                      # returns 'days' for $dc >  1
+
+  # --------------------------------------------------------------------------
+
   # find all *.txt files in all subdirectories starting from /usr/local
   # returned files are with full path names
   my @files = glob_tree( '/usr/local/*.txt' );
@@ -666,6 +729,13 @@ keys (possibly key paths like 'KEY1/KEY2/KEY3'):
 
   (more docs)
 
+=head1 DATA::TOOLS SUB-MODULES
+
+Data::Tools package includes several sub-modules:
+
+  * Data::Tools::Socket (socket I/O processing, TODO: docs)
+  * Data::Tools::Time   (time processing)
+
 =head1 REQUIRED MODULES
 
 Data::Tools is designed to be simple, compact and self sufficient. 
